package net.wanji.utc.hisense.cache;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.dao.mapper.ManufacturerInfoMapper;
import net.wanji.databus.po.CrossInfoPO;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author duanruiming
 * @date 2023/05/09 14:30
 */
@Slf4j
@Component
@SuppressWarnings("unchecked")
public class CrossInfoCache implements ApplicationRunner {

    @Resource
    private CrossInfoMapper crossInfoMapper;
    @Resource
    private ManufacturerInfoMapper manufacturerInfoMapper;

    private static final Map<String, CrossInfoPO> crossInfoMap = new HashMap<>();

    public static Map<String, CrossInfoPO> getCrossInfoCache() {
        if (!crossInfoMap.isEmpty()) {
            return crossInfoMap;
        }
        return Collections.EMPTY_MAP;
    }

    public static CrossInfoPO getCrossInfoBySignalCode(String code) {
        if (!crossInfoMap.isEmpty()) {
            for (Map.Entry<String, CrossInfoPO> entry : crossInfoMap.entrySet()) {
                CrossInfoPO value = entry.getValue();
                if (StringUtils.equalsIgnoreCase(code, value.getCode())) {
                    return value;
                }
            }
        }
        return null;
    }

    public static CrossInfoPO getCrossInfoByIp(String ip) {
        if (!crossInfoMap.isEmpty()) {
            for (Map.Entry<String, CrossInfoPO> entry : crossInfoMap.entrySet()) {
                CrossInfoPO crossInfoPO = entry.getValue();
                if (StringUtils.equalsIgnoreCase(crossInfoPO.getIp(), ip)) {
                    return crossInfoPO;
                }
            }
        }
        return null;
    }

    @Override
    public void run(ApplicationArguments args) throws Exception {
        init();
    }

    private void init() {
        Integer manufacturerId = manufacturerInfoMapper.selectIdByCode(BaseEnum.VendorTypeEnum.HS.getNick());
        List<CrossInfoPO> CrossInfoPOS = crossInfoMapper.selectAll();
        for (CrossInfoPO crossInfoPO : CrossInfoPOS) {
            if (Objects.nonNull(manufacturerId) && Objects.equals(manufacturerId, crossInfoPO.getManufacturerId())) {
                crossInfoMap.put(crossInfoPO.getId(), crossInfoPO);
            }
        }
    }

}
