package net.wanji.utc.hisense.cache;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.CrossSchedulesPO;
import net.wanji.databus.dao.entity.CrossSchemePO;
import net.wanji.databus.dao.entity.CrossSectionPO;
import net.wanji.databus.dao.mapper.CrossSchedulesMapper;
import net.wanji.databus.dao.mapper.CrossSchemeMapper;
import net.wanji.databus.dao.mapper.CrossSectionMapper;
import net.wanji.databus.po.CrossInfoPO;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/09/23 17:26
 */
@Slf4j
@Component
@SuppressWarnings("unchecked")
@RequiredArgsConstructor
public class CrossRunSchemeCache implements CommandLineRunner {

    public static final Map<String, String> currentRunSchemeNoCache = new HashMap<>();

    @Resource
    private CrossSchedulesMapper crossSchedulesMapper;
    @Resource
    private CrossSectionMapper crossSectionMapper;
    @Resource
    private CrossSchemeMapper crossSchemeMapper;
    @Override
    public void run(String... args) throws Exception {
        init();
    }

    public void init() throws Exception {
        Map<String, CrossInfoPO> crossInfoCache = CrossInfoCache.getCrossInfoCache();

        if (!crossInfoCache.isEmpty()) {
            for (Map.Entry<String, CrossInfoPO> entry : crossInfoCache.entrySet()) {
                String crossId = entry.getKey();
                List<CrossSectionPO> crossSectionPOS = crossSectionMapper.selectByCrossId(crossId);
                List<CrossSchedulesPO> crossSchedulesPOS = crossSchedulesMapper.selectByCrossIds(Arrays.asList(crossId));
                if (!CollectionUtils.isEmpty(crossSchedulesPOS)) {
                    Map<Integer, List<CrossSchedulesPO>> schedulesMap = crossSchedulesPOS.stream().collect(Collectors.groupingBy(CrossSchedulesPO::getPlanId));
                    // 判断当前执行的调度，获取日计划号
                    Date date = new Date();
                    LocalTime currentTime = LocalTime.parse(DateUtil.getTime());
                    Integer currentPlanId = getCurrentPlanId(schedulesMap, date);
                    Integer schemeId = getCurrentSchemeId(crossSectionPOS, currentTime, currentPlanId);
                    CrossSchemePO crossSchemePO = crossSchemeMapper.selectSchemePOById(schemeId);
                    if (Objects.nonNull(crossSchemePO)) {
                        currentRunSchemeNoCache.put(crossId, crossSchemePO.getSchemeNo());
                    }
                }
            }
        }
    }

    private static Integer getCurrentSchemeId(List<CrossSectionPO> crossSectionPOS, LocalTime currentTime, Integer currentPlanId) {
        if (!CollectionUtils.isEmpty(crossSectionPOS)) {
            List<CrossSectionPO> currentPlanIdSectionList = crossSectionPOS.stream().filter(po -> Objects.equals(currentPlanId, po.getPlanId())).collect(Collectors.toList());
            for (CrossSectionPO crossSectionPO : currentPlanIdSectionList) {
                Integer schemeId = crossSectionPO.getSchemeId();
                LocalTime startTime = LocalTime.parse(crossSectionPO.getStartTime());
                LocalTime endTime = LocalTime.parse(crossSectionPO.getEndTime());
                // 过滤非当前时段数据
                if (currentTime.isAfter(startTime) && currentTime.isBefore(endTime)) {
                    return schemeId;
                }
            }
        }
        return 0;
    }

    private static Integer getCurrentPlanId(Map<Integer, List<CrossSchedulesPO>> schedulesMap, Date currentDate) {
        for (Map.Entry<Integer, List<CrossSchedulesPO>> entry : schedulesMap.entrySet()) {
            Integer planId = entry.getKey();
            List<CrossSchedulesPO> schedulesPOS = entry.getValue();
            if (!CollectionUtils.isEmpty(schedulesPOS)) {
                for (CrossSchedulesPO schedulesPO : schedulesPOS) {
                    Integer isSpectialDay = schedulesPO.getWeek(); // 0 为特殊日期
                    Date specialDate = schedulesPO.getSpecialDate();
                    String specialDate4DB = DateUtil.format(specialDate, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND);
                    String currentDateStr = DateUtil.format(currentDate, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND);
                    if (Objects.equals(0, isSpectialDay)) {
                        if (StringUtils.equalsIgnoreCase(currentDateStr.substring(0, 10), specialDate4DB.substring(0, 10))) {
                            return planId;
                        }
                    }
                    Integer week4DB = schedulesPO.getWeek();
                    int weekCurrent = DateUtil.getWeek(currentDate);
                    if (week4DB == weekCurrent) {
                        return planId;
                    }
                }
            }
        }
        return 0;
    }
}
