package net.wanji.utc.hisense.cache;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.entity.CrossPhasePO;
import net.wanji.databus.dao.entity.CrossSchemePO;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossPhaseMapper;
import net.wanji.databus.dao.mapper.CrossSchemeMapper;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.utc.hisense.pojo.dto.CrossSchemePhaseCountDownDTO;
import net.wanji.utc.hisense.pojo.dto.PhaseCountDownDTO;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/09/22 14:19
 */
@Slf4j
@Component
@SuppressWarnings("unchecked")
@RequiredArgsConstructor
public class CrossSchemePhaseTimeCountCache implements CommandLineRunner {

    public static final List<CrossSchemePhaseCountDownDTO> crossSchemePhaseCountDownList = new ArrayList<>();
    @Resource
    private CrossPhaseMapper crossPhaseMapper;
    @Resource
    private CrossSchemeMapper crossSchemeMapper;
    @Resource
    private CrossInfoMapper crossInfoMapper;

    @Override
    public void run(String... args) throws Exception {
        try {
            init();
        } catch (Exception e) {
            log.error("路口方案相位倒计时缓存加载失败", e);
            throw new Exception(e);
        }
    }

    public void init() throws Exception {
        List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectAll();
        if (!CollectionUtils.isEmpty(crossInfoPOList)) {
            for (CrossInfoPO crossInfoPO : crossInfoPOList) {
                String crossId = crossInfoPO.getId();
                List<CrossPhasePO> crossPhasePOS = crossPhaseMapper.selectByCrossId(crossId);
                if (!CollectionUtils.isEmpty(crossPhasePOS)) {
                    Map<Integer, List<CrossPhasePO>> schemeIdListMap = crossPhasePOS.stream().collect(Collectors.groupingBy(CrossPhasePO::getPlanId));
                    for (Map.Entry<Integer, List<CrossPhasePO>> entry : schemeIdListMap.entrySet()) {
                        Integer schemeId = entry.getKey();
                        CrossSchemePhaseCountDownDTO crossSchemePhaseCountDownDTO = new CrossSchemePhaseCountDownDTO();
                        CrossSchemePO crossSchemePO = crossSchemeMapper.selectSchemePOById(schemeId);
                        if (Objects.isNull(crossSchemePO)) {
                            continue;
                        }
                        Integer cycle = crossSchemePO.getCycle();
                        String schemeNo = crossSchemePO.getSchemeNo();
                        List<CrossPhasePO> schemeNoList = entry.getValue();
                        if (!CollectionUtils.isEmpty(schemeNoList)) {
                            List<CrossPhasePO> phaseSortList = schemeNoList.stream().sorted(Comparator.comparingInt(CrossPhasePO::getSort)).collect(Collectors.toList());
                            Integer changePhaseTime = 0;
                            List<PhaseCountDownDTO> phaseCountDownDTOS = new ArrayList<>(phaseSortList.size());
                            for (CrossPhasePO crossPhasePO : phaseSortList) {
                                String phaseNo = crossPhasePO.getPhaseNo();
                                Integer phaseTime = crossPhasePO.getPhaseTime();
                                Integer yellowTime = crossPhasePO.getYellowTime();
                                Integer redTime = crossPhasePO.getRedTime();
                                changePhaseTime += phaseTime;

                                PhaseCountDownDTO phaseCountDownDTO = new PhaseCountDownDTO();
                                phaseCountDownDTO.setPhaseNo(phaseNo);
                                phaseCountDownDTO.setPhaseTime(phaseTime);
                                phaseCountDownDTO.setChangePhaseTime(changePhaseTime);
                                phaseCountDownDTO.setYellowTime(yellowTime);
                                phaseCountDownDTO.setRedTime(redTime);
                                phaseCountDownDTOS.add(phaseCountDownDTO);
                            }
                            crossSchemePhaseCountDownDTO.setPhaseCountDownDTOList(phaseCountDownDTOS);
                        }
                        crossSchemePhaseCountDownDTO.setCrossId(crossId);
                        crossSchemePhaseCountDownDTO.setSchemeNo(schemeNo);
                        crossSchemePhaseCountDownList.add(crossSchemePhaseCountDownDTO);
                    }
                }
            }
        }
    }
}
