package net.wanji.utc.hisense.pojo.result;

import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

/**
 * @ClassName CrossSchemeRings
 * @Description 配时方案环图
 * @Author zhouleilei
 * @Date 2024/11/4 20:04
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class CrossSchemeRings {
    /**
     * 控制点编号
     */
    private String spot;
    /**
     * 方案号
     */
    private String pattern;
    /**
     * 周期长，单位秒
     */
    private String cycleLen;
    /**
     * 方案中环的个数
     */
    private String ringCount;
    /**
     * 相位差
     */
    private String offset;
    /**
     * 环中相位的顺序，i 表示环号。相位号之间用空格隔开,屏障之间用下划线”_”分隔。
     */
    private JSONObject cycleList;
    /**
     * 相位列表
     */
    private List<Phase> phaseList;

    @Data
    public static class  Phase{
        /**
         * 16 个相位的红灯时间（单位秒）,中间用空格隔开,不存在的相位置 0
         */
        private String redTime;
        /**
         * 16 个相位的黄灯时间（单位秒）,中间用空格隔开,不存在的相位置 0
         */
        private String yellowTime;
        /**
         * 16 个相位的绿灯时间（单位秒）,中间用空格隔开,不存在的相位置 0
         */
        private String greenTime;
        /**
         * 16 个相位的方向,中间用空格隔开,不存在的相位置 0。每个相位由 8 字节整型值存储（最多支持 7 个放行方向）
         */
        private String direction;
        /**
         * 16 个相位的最小绿，单位秒；最小绿表示相位绿灯时间可调整的最小值，优化控制的最小绿灯时长受该参数限制
         */
        private String minGTime;
        /**
         * 16 个相位的最大绿，单位秒；最大绿表示相位绿灯时间可调整的最大值，优化控制暂时不受该参数限制
         */
        private String maxGTime;
        /**
         * 16 个相位的放行方向，每个相位由 8 字节整型值存储（最多支持 7 个放行方向），每个字节表示一个放行方向
         */
        private String channelDim;
        /**
         * 16 个相位的并发相位，相位号按位存储，每个比特位位置代表一个可并发的相位号，例 48 代表相位 5、6，192 代表相位 7、8
         */
        private String concurrent;

    }

}
