package net.wanji.utc.hisense.service;

import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dto.ExtendPhaseDTO;
import net.wanji.databus.dto.StepingPhaseDTO;
import net.wanji.databus.vo.TempSchemeSendVO;
import net.wanji.databus.vo.*;

/**
 * @author duanruiming
 * @date 2023/05/08 10:42
 */
public interface ControlCommandService {

    /**
     * 方案下发
     */
    JsonViewObject schemeSend(SchemeSendVO schemeSendVO) throws Exception;

    /**
     * 计划下发
     *
     * @param planSendVO 计划下发VO
     * @return
     */
    JsonViewObject planSend(PlanSendVO planSendVO) throws Exception;

    /**
     * 时间表下发
     *
     * @param scheduleSendVO 时间表下发VO
     * @return
     */
    JsonViewObject scheduleSend(ScheduleSendVO scheduleSendVO) throws Exception;

    /**
     * 锁定控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    JsonViewObject lockControl(ControlCommandVO commandVO) throws Exception;

    /**
     * 步进控制
     *
     * @param code    信号机编号
     * @param command 1 开始步进 0 取消步进
     * @param stepNum 0 顺序步进 n 跳过n个相位
     * @return
     * @throws Exception 异常
     */
    JsonViewObject stepControl(String code, Integer command, Integer stepNum) throws Exception;

    /**
     * 设置信号机控制模式
     *
     * @param commandVO        信号机编号
     * @return
     * @throws Exception 异常
     */
    JsonViewObject setSignalControl(ControlCommandVO commandVO) throws Exception;

    /**
     * 恢复时间表
     *
     * @param code 信号机编号
     * @return
     */
    JsonViewObject recoverSchedule(String code) throws Exception;

    /**
     * 相位配时下发
     *
     * @return
     * @throws InterruptedException 中断异常
     */
    JsonViewObject phaseTimingSend(PhaseTimingSendVO phaseTimingSendVO) throws Exception;

    /**
     * 相位差下发
     */
    JsonViewObject phaseDiffSend();

    /**
     * 删除信号机配置参数
     * 删除信号机配置参数-目前支持相位、方案、计划、日期
     *
     * @param delBaseConfigPO
     * @return
     */
//    JsonViewObject delBaseConfig(DelBaseConfigPOfigPO delBaseConfigPO) throws Exception;

    /**
     * 临时方案下发
     */
    JsonViewObject tempSchemeDetail(TempSchemeSendVO tempSchemeSendVO) throws Exception;

    /**
     * @Description 延长相位
     * @param extendPhaseDTO
     * @return
     **/
    JsonViewObject extendPhase(ExtendPhaseDTO extendPhaseDTO) throws Exception;

    /**
     * @Description 步进相位
     * @param stepingPhaseDTO
     * @return
     **/
    JsonViewObject stepingPhase(StepingPhaseDTO stepingPhaseDTO) throws Exception;

}
