package net.wanji.utc.hisense.service.impl;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dto.ExtendPhaseDTO;
import net.wanji.databus.dto.StepingPhaseDTO;
import net.wanji.databus.po.CrossInfoPOExt;
import net.wanji.databus.vo.TempSchemeSendVO;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.vo.*;
import net.wanji.utc.hisense.common.OperationBaseDom;
import net.wanji.utc.hisense.common.constants.HttpConstants;
import net.wanji.utc.hisense.service.ControlCommandService;
import net.wanji.utc.hisense.util.OkHttpClientUtil;
import org.apache.commons.lang3.ObjectUtils;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * @author duanruiming
 * @date 2023/05/08 10:43
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class ControlCommandServiceImpl implements ControlCommandService {
    @Value("${hisense.url}")
    private String hisenseUrl;
    @Resource
    private CrossInfoMapper crossInfoMapper;

    @Override
    public JsonViewObject schemeSend(SchemeSendVO schemeSendVO) throws Exception {
        // 海信方案下发
        // 下发xml
        return null;
    }

    @Override
    public JsonViewObject planSend(PlanSendVO planSendVO) throws Exception {
        return null;
    }

    @Override
    public JsonViewObject scheduleSend(ScheduleSendVO scheduleSendVO) throws Exception {
        return null;
    }

    @Override
    public JsonViewObject lockControl(ControlCommandVO commandVO) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(commandVO.getCrossCode());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_22);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.CTRLSTEP).setText("0");
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        if (commandVO.getCommand() == 1) {
            //锁相
            messageContent.addElement(HttpConstants.COMMAND).setText("1");
        } else if (commandVO.getCommand() == 0) {
            //解锁
            messageContent.addElement(HttpConstants.COMMAND).setText("0");
        }
        //相位驻留
//            String xmlPost = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        String xmlPost = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                "    <subSystem>Hisense</subSystem>\n" +
                "    <messageType>12</messageType>\n" +
                "    <isRequest>0</isRequest>\n" +
                "    <seq>20241103155205549244</seq>\n" +
                "    <needResponse>1</needResponse>\n" +
                "    <result>1</result>\n" +
                "    <flag>0</flag>\n" +
                "    <tStamp>1731037114609</tStamp>\n" +
                "    <messageContent>\n" +
                "        <Spot>311358</Spot>\n" +
                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                "    </messageContent>\n" +
                "</systemScription>";
        if (StringUtils.isNotBlank(xmlPost)) {
            Document extendPost = DocumentHelper.parseText(xmlPost);
            String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
            if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                return jsonViewObject.success("相位锁定/解锁 - 控制成功");
            }else {
                return jsonViewObject.fail("相位锁定/解锁 - 控制失败");
            }
        }
        return jsonViewObject.fail("相位锁定/解锁 - 控制失败");
    }

    @Override
    public JsonViewObject stepControl(String code, Integer command, Integer stepNum) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(code);
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_22);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.COMMAND).setText(command + "");
        //目前海信步进幅度只支持0
        messageContent.addElement(HttpConstants.CTRLSTEP).setText("0");
        if (stepNum == 0) {
            //顺序步进，循环一次
            stepNum = 1;
        }
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (command == 0) {
            //取消步进
//            boolean isOk = sendMesIsOk(document.asXML());
            boolean isOk = true;
            if (isOk) {
                return jsonViewObject.success("取消步进 - 控制成功");
            } else {
                return jsonViewObject.fail("取消步进 - 控制失败");
            }
        } else {
            //相位驻留
//            String xmlPost = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
            String xmlPost = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                    "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                    "    <subSystem>Hisense</subSystem>\n" +
                    "    <messageType>12</messageType>\n" +
                    "    <isRequest>0</isRequest>\n" +
                    "    <seq>20241103155205549244</seq>\n" +
                    "    <needResponse>1</needResponse>\n" +
                    "    <result>1</result>\n" +
                    "    <flag>0</flag>\n" +
                    "    <tStamp>1731037114609</tStamp>\n" +
                    "    <messageContent>\n" +
                    "        <Spot>311358</Spot>\n" +
                    "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                    "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                    "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                    "    </messageContent>\n" +
                    "</systemScription>";
            if (StringUtils.isNotBlank(xmlPost)) {
                Document extendPost = DocumentHelper.parseText(xmlPost);
                String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                    //开始步进
                    //给海信发送http请求
                    for (int i = 0; i < stepNum; i++) {
//                        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
                        String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                                "    <subSystem>Hisense</subSystem>\n" +
                                "    <messageType>12</messageType>\n" +
                                "    <isRequest>0</isRequest>\n" +
                                "    <seq>20241103155205549244</seq>\n" +
                                "    <needResponse>1</needResponse>\n" +
                                "    <result>1</result>\n" +
                                "    <flag>0</flag>\n" +
                                "    <tStamp>1731037114609</tStamp>\n" +
                                "    <messageContent>\n" +
                                "        <Spot>311358</Spot>\n" +
                                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                                "    </messageContent>\n" +
                                "</systemScription>";
                        if (StringUtils.isNotBlank(post)) {
                            Document postResult = DocumentHelper.parseText(post);
                            String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                            if (HttpConstants.RESULT_1.equals(result) || HttpConstants.RESULT_2.equals(result)) {
                                log.info("路口号 ：{}，第 {} 次步进成功,result:{}", code, i + 1, result);
                            } else {
                                log.error("路口号 ：{}，第 {} 次步进失败,result:{}", code, i + 1, result);
                                messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                                //取消步进
//                                boolean isOk = sendMesIsOk(document.asXML());
                                boolean isOk = true;
                                if (isOk) {
                                    return jsonViewObject.fail("相位步进失败，取消步进成功，路口号： " + code);
                                } else {
                                    return jsonViewObject.fail("相位步进失败，取消步进失败，路口号： " + code);
                                }
                            }
                            //停顿10ms
                            Thread.sleep(10);
                        } else {
                            messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                            //取消步进
//                            boolean isOk = sendMesIsOk(document.asXML());
                            boolean isOk = true;
                            if (isOk) {
                                return jsonViewObject.fail("相位步进失败，取消步进成功，路口号： " + code);
                            } else {
                                return jsonViewObject.fail("相位步进失败，取消步进失败，路口号： " + code);
                            }
                        }
                    }
                }
            } else {
                return jsonViewObject.fail("相位步进 - 控制失败");
            }
        }
        return jsonViewObject.fail("相位步进 - 控制失败");
    }

    @Override
    public JsonViewObject setSignalControl(ControlCommandVO commandVO) throws Exception {
        return null;
    }

    @Override
    public JsonViewObject recoverSchedule(String code) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(code);
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_16);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.PATTERN).setText("0");

        //给海信发送http请求
//        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                "    <subSystem>Hisense</subSystem>\n" +
                "    <messageType>12</messageType>\n" +
                "    <isRequest>0</isRequest>\n" +
                "    <seq>20241103155205549244</seq>\n" +
                "    <needResponse>1</needResponse>\n" +
                "    <result>1</result>\n" +
                "    <flag>0</flag>\n" +
                "    <tStamp>1731037114609</tStamp>\n" +
                "    <messageContent>\n" +
                "        <Spot>311358</Spot>\n" +
                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                "    </messageContent>\n" +
                "</systemScription>";
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (StringUtils.isBlank(post)) {
            log.error("5.10. 手动方案控制- 恢复固定配时 失败");
            return jsonViewObject.fail("5.10. 手动方案控制- 恢复固定配时 失败");
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();

        if (HttpConstants.RESULT_1.equals(result)) {
            //指令下发成功
            jsonViewObject.success("恢复固定配置 - 指令下发成功");
        } else if (HttpConstants.RESULT_0.equals(result)) {
            //指令下发失败
            jsonViewObject.fail("恢复固定配置 - 指令下发失败");
        } else if (HttpConstants.RESULT_2.equals(result)) {
            //指令执行成功
            jsonViewObject.success("恢复固定配置 - 指令执行成功");
        } else {
            //指令执行失败
            jsonViewObject.fail("恢复固定配置 - 指令执行失败");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject phaseTimingSend(PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        //TODO 海信相位配时下发
        return null;
    }

    @Override
    public JsonViewObject phaseDiffSend() {
        return null;
    }

    @Override
    public JsonViewObject tempSchemeDetail(TempSchemeSendVO tempSchemeSendVO) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(tempSchemeSendVO.getCrossId());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_39);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.PATTERN).setText(tempSchemeSendVO.getPattern());
        messageContent.addElement(HttpConstants.CYCLE).setText(tempSchemeSendVO.getCycle());
        //如果为空赋值神思的默认值
        if (ObjectUtils.isEmpty(tempSchemeSendVO.getOffset())) {
            tempSchemeSendVO.setOffset("0");
        }
        if (ObjectUtils.isEmpty(tempSchemeSendVO.getType())) {
            tempSchemeSendVO.setType("1");
        }
        if (ObjectUtils.isEmpty(tempSchemeSendVO.getSplit())) {
            tempSchemeSendVO.setSplit("40,0,0,40,50,50,0,0,0,0,0,0,0,0,0,0");
        }
        messageContent.addElement(HttpConstants.OFFSET).setText(tempSchemeSendVO.getOffset());
        messageContent.addElement(HttpConstants.TYPE).setText(tempSchemeSendVO.getType());
        messageContent.addElement(HttpConstants.SPLIT).setText(tempSchemeSendVO.getSplit());

        //给海信发送http请求
//        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                "    <subSystem>Hisense</subSystem>\n" +
                "    <messageType>12</messageType>\n" +
                "    <isRequest>0</isRequest>\n" +
                "    <seq>20241103155205549244</seq>\n" +
                "    <needResponse>1</needResponse>\n" +
                "    <result>1</result>\n" +
                "    <flag>0</flag>\n" +
                "    <tStamp>1731037114609</tStamp>\n" +
                "    <messageContent>\n" +
                "        <Spot>311358</Spot>\n" +
                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                "    </messageContent>\n" +
                "</systemScription>";
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (StringUtils.isBlank(post)) {
            log.error("HTTP请求 5.16. 方案优化控制 失败");
            return jsonViewObject.fail("HTTP请求 5.16. 方案优化控制 失败");
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();

        if (HttpConstants.RESULT_1.equals(result)) {
            //指令下发成功
            jsonViewObject.success("临时方案 - 指令下发成功");
        } else if (HttpConstants.RESULT_0.equals(result)) {
            //指令下发失败
            jsonViewObject.fail("临时方案 - 指令下发失败");
        } else if (HttpConstants.RESULT_2.equals(result)) {
            //指令执行成功
            jsonViewObject.success("临时方案 - 指令执行成功");
        } else {
            //指令执行失败
            jsonViewObject.fail("临时方案 - 指令执行失败");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject extendPhase(ExtendPhaseDTO extendPhaseDTO) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(extendPhaseDTO.getCrossId());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_22);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        //暂时按照神思的写死
        messageContent.addElement(HttpConstants.COMMAND).setText("1");
        messageContent.addElement(HttpConstants.CTRLSTEP).setText("0");

        //给海信发送http请求
//        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                "    <subSystem>Hisense</subSystem>\n" +
                "    <messageType>12</messageType>\n" +
                "    <isRequest>0</isRequest>\n" +
                "    <seq>20241103155205549244</seq>\n" +
                "    <needResponse>1</needResponse>\n" +
                "    <result>1</result>\n" +
                "    <flag>0</flag>\n" +
                "    <tStamp>1731037114609</tStamp>\n" +
                "    <messageContent>\n" +
                "        <Spot>311358</Spot>\n" +
                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                "    </messageContent>\n" +
                "</systemScription>";
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (StringUtils.isBlank(post)) {
            log.error("HTTP请求 5.12. 步进及取消控制 失败");
            return jsonViewObject.fail("HTTP请求 5.12. 步进及取消控制 失败");
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        //解除步进
        messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
        if (HttpConstants.RESULT_1.equals(result) || HttpConstants.RESULT_2.equals(result)) {
            //执行成功，开始延长
            Thread.sleep(extendPhaseDTO.getExtendTime() * 1000);
            //给海信发送http请求
//            String post1 = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
            String post1 = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                    "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                    "    <subSystem>Hisense</subSystem>\n" +
                    "    <messageType>12</messageType>\n" +
                    "    <isRequest>0</isRequest>\n" +
                    "    <seq>20241103155205549244</seq>\n" +
                    "    <needResponse>1</needResponse>\n" +
                    "    <result>1</result>\n" +
                    "    <flag>0</flag>\n" +
                    "    <tStamp>1731037114609</tStamp>\n" +
                    "    <messageContent>\n" +
                    "        <Spot>311358</Spot>\n" +
                    "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                    "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                    "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                    "    </messageContent>\n" +
                    "</systemScription>";
            if (StringUtils.isNotBlank(post1)) {
                Document extendPost = DocumentHelper.parseText(post1);
                String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                    return jsonViewObject.success("延长相位 - 控制成功");
                }
            }
            //延长失败,解除步进控制,发起三次取消控制请求
            log.error("HTTP请求 5.12. 步进及取消控制 失败");
            //重新发起三次取消控制请求
            for (int j = 0; j < 3; j++) {
//                String httpPost = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
                String httpPost = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                        "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                        "    <subSystem>Hisense</subSystem>\n" +
                        "    <messageType>12</messageType>\n" +
                        "    <isRequest>0</isRequest>\n" +
                        "    <seq>20241103155205549244</seq>\n" +
                        "    <needResponse>1</needResponse>\n" +
                        "    <result>1</result>\n" +
                        "    <flag>0</flag>\n" +
                        "    <tStamp>1731037114609</tStamp>\n" +
                        "    <messageContent>\n" +
                        "        <Spot>311358</Spot>\n" +
                        "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                        "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                        "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                        "    </messageContent>\n" +
                        "</systemScription>";
                if (StringUtils.isNotBlank(httpPost)) {
                    Document extendPost = DocumentHelper.parseText(httpPost);
                    String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                    if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                        return jsonViewObject.success("延长相位 - 控制成功");
                    }
                }
                Thread.sleep(1000);
            }

        } else {
            //重新发起三次取消控制请求
            for (int j = 0; j < 3; j++) {
                String httpPost = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
                if (StringUtils.isNotBlank(httpPost)) {
                    Document extendPost = DocumentHelper.parseText(httpPost);
                    String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                    if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                        return jsonViewObject.success("延长相位 - 控制成功");
                    }
                }
                Thread.sleep(1000);
            }
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject stepingPhase(StepingPhaseDTO stepingPhaseDTO) throws Exception {
        if (ObjectUtils.isEmpty(stepingPhaseDTO.getStepCount()) || stepingPhaseDTO.getStepCount() == 0) {
            //默认步进1次
            stepingPhaseDTO.setStepCount(1);
        }
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(stepingPhaseDTO.getCrossId());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_22);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.COMMAND).setText(stepingPhaseDTO.getCommand() + "");
        //海信现在只支持0
        messageContent.addElement(HttpConstants.CTRLSTEP).setText("0");
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (stepingPhaseDTO.getCommand() == 0) {
            //取消步进
//            boolean isOk = sendMesIsOk(document.asXML());
            boolean isOk = true;
            if (true) {
                return jsonViewObject.success("取消步进 - 控制成功");
            } else {
                return jsonViewObject.fail("取消步进 - 控制失败");
            }
        } else if (stepingPhaseDTO.getCommand() == 1) {
            //逻辑：第一次发送步进指令是相位驻留，以后发送才是相位步进，步进成功则取消步进。步进失败，也立即下发取消步进控制
            //相位驻留
//            String xmlPost = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
            String xmlPost = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                    "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                    "    <subSystem>Hisense</subSystem>\n" +
                    "    <messageType>12</messageType>\n" +
                    "    <isRequest>0</isRequest>\n" +
                    "    <seq>20241103155205549244</seq>\n" +
                    "    <needResponse>1</needResponse>\n" +
                    "    <result>1</result>\n" +
                    "    <flag>0</flag>\n" +
                    "    <tStamp>1731037114609</tStamp>\n" +
                    "    <messageContent>\n" +
                    "        <Spot>311358</Spot>\n" +
                    "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                    "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                    "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                    "    </messageContent>\n" +
                    "</systemScription>";

            if (StringUtils.isNotBlank(xmlPost)) {
                Document extendPost = DocumentHelper.parseText(xmlPost);
                String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                    //开始步进
                    //给海信发送http请求
                    for (int i = 0; i < stepingPhaseDTO.getStepCount(); i++) {
//                        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
                        String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                                "    <subSystem>Hisense</subSystem>\n" +
                                "    <messageType>12</messageType>\n" +
                                "    <isRequest>0</isRequest>\n" +
                                "    <seq>20241103155205549244</seq>\n" +
                                "    <needResponse>1</needResponse>\n" +
                                "    <result>1</result>\n" +
                                "    <flag>0</flag>\n" +
                                "    <tStamp>1731037114609</tStamp>\n" +
                                "    <messageContent>\n" +
                                "        <Spot>311358</Spot>\n" +
                                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                                "    </messageContent>\n" +
                                "</systemScription>";
                        if (StringUtils.isNotBlank(post)) {
                            Document postResult = DocumentHelper.parseText(post);
                            String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                            if (HttpConstants.RESULT_1.equals(result) || HttpConstants.RESULT_2.equals(result)) {
                                log.info("路口号 ：{}，第 {} 次步进成功,result:{}", stepingPhaseDTO.getCrossId(), i + 1, result);
                            } else {
                                log.error("路口号 ：{}，第 {} 次步进失败,result:{}", stepingPhaseDTO.getCrossId(), i + 1, result);
                                messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                                //取消步进
//                                boolean isOk = sendMesIsOk(document.asXML());
                                boolean isOk = true;
                                if (isOk) {
                                    return jsonViewObject.fail("相位步进失败，取消步进成功，路口号： " + stepingPhaseDTO.getCrossId());
                                } else {
                                    return jsonViewObject.fail("相位步进失败，取消步进失败，路口号： " + stepingPhaseDTO.getCrossId());
                                }
                            }
                            //停顿10ms
                            Thread.sleep(10);
                        } else {
                            messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                            //取消步进
//                            boolean isOk = sendMesIsOk(document.asXML());
                            boolean isOk = true;
                            if (isOk) {
                                return jsonViewObject.fail("相位步进失败，取消步进成功，路口号： " + stepingPhaseDTO.getCrossId());
                            } else {
                                return jsonViewObject.fail("相位步进失败，取消步进失败，路口号： " + stepingPhaseDTO.getCrossId());
                            }
                        }

                    }
                    messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                    //取消步进
//                    boolean isOk = sendMesIsOk(document.asXML());
                    boolean isOk = true;
                    if (isOk) {
                        return jsonViewObject.success("相位步进成功，，路口号： " + stepingPhaseDTO.getCrossId());
                    } else {
                        return jsonViewObject.fail("相位步进成功，但取消步进失败，路口号： " + stepingPhaseDTO.getCrossId());
                    }
                }
            } else {
                return jsonViewObject.fail("相位步进 - 控制失败");
            }
        }
        return jsonViewObject.fail("相位步进 - 控制失败");
    }

    private boolean sendMesIsOk(String xml) throws DocumentException, InterruptedException {
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, xml);
        if (StringUtils.isBlank(post)) {
            return false;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result) || HttpConstants.RESULT_2.equals(result)) {
            return true;
        } else {
            //连续重试发三次
            for (int i = 0; i < 3; i++) {
                post = OkHttpClientUtil.xmlPost(hisenseUrl, xml);
                postResult = DocumentHelper.parseText(post);
                result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                if (HttpConstants.RESULT_1.equals(result) || HttpConstants.RESULT_2.equals(result)) {
                    return true;
                }
                Thread.sleep(1000);
            }
        }
        return false;
    }
}
