package net.wanji.utc.hisense.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.ScheduleUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.CrossLightsPO;
import net.wanji.databus.dao.entity.CrossPlanPO;
import net.wanji.databus.dao.entity.CrossSchedulesPO;
import net.wanji.databus.dao.entity.CrossSectionPO;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.dao.mapper.ManufacturerInfoMapper;
import net.wanji.databus.dto.*;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.CrossInfoPOExt;
import net.wanji.databus.vo.PlanSectionVO;
import net.wanji.databus.vo.SchemePhaseLightsVO;
import net.wanji.utc.hisense.cache.CrossInfoCache;
import net.wanji.utc.hisense.common.ChannelControlEnum;
import net.wanji.utc.hisense.common.HisenseTurnEnum;
import net.wanji.utc.hisense.common.HiseseDirEnum;
import net.wanji.utc.hisense.common.OperationBaseDom;
import net.wanji.utc.hisense.common.constants.HttpConstants;
import net.wanji.utc.hisense.mapper.*;
import net.wanji.utc.hisense.pojo.result.CoordinationStatus;
import net.wanji.utc.hisense.pojo.result.CrossSchemeRings;
import net.wanji.utc.hisense.pojo.view.VIntersectionView;
import net.wanji.utc.hisense.pojo.view.VNtcipChannelPhaseDTO;
import net.wanji.utc.hisense.pojo.view.VNtcipTimeBaseDayplanDTO;
import net.wanji.utc.hisense.pojo.view.VNtcipTimeBaseScheduleView;
import net.wanji.utc.hisense.service.StaticInfoService;
import net.wanji.utc.hisense.util.OkHttpClientUtil;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 17:29
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class StaticInfoServiceImpl implements StaticInfoService {

    @Value("${hisense.url}")
    private String hisenseUrl;
    @Resource
    private CrossInfoMapper crossInfoMapper;
    @Resource
    private ManufacturerInfoMapper manufacturerInfoMapper;
    @Resource
    private VIntersectionMapper vIntersectionMapper;
    @Resource
    private VNtcipTimeBaseScheduleMapper vNtcipTimeBaseScheduleMapper;
    @Resource
    private VNtcipTimeBaseDayplanMapper vNtcipTimeBaseDayplanMapper;
    @Resource
    private VNtcipTimeBaseAscActionMapper vNtcipTimeBaseAscActionMapper;
    @Resource
    private VNtcipChannelViewMapper vNtcipChannelViewMapper;


    @Override
    public List<CrossInfoPO> crossBasicInfo(CrossInfoDTO crossInfoDTO) throws Exception {
        Integer id = manufacturerInfoMapper.selectIdByCode(crossInfoDTO.getManufacturerCode());
        List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectByManufacturerId(id);
        List<String> codeList = crossInfoPOList.stream().map(CrossInfoPO::getCode).collect(Collectors.toList());
        Map<String, String> codeCrossIdMap = crossInfoPOList.stream().collect(Collectors.toMap(CrossInfoPO::getCode, CrossInfoPO::getId));
        List<CrossInfoPO> crossInfoPOS = new ArrayList<>();
        if (StringUtils.equalsIgnoreCase("HISENSE", crossInfoDTO.getManufacturerCode())) {
            LambdaQueryWrapper<VIntersectionView> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.in(VIntersectionView::getCIntsID, codeList);
            List<VIntersectionView> vIntersectionEntities = vIntersectionMapper.selectList(queryWrapper);
            for (VIntersectionView vIntersectionView : vIntersectionEntities) {
                String cIntsID = vIntersectionView.getCIntsID();
                CrossInfoPO crossInfoPO = new CrossInfoPO();
                crossInfoPO.setId(codeCrossIdMap.get(cIntsID));
                crossInfoPO.setCode(cIntsID);
                crossInfoPO.setName(vIntersectionView.getCIntsName());
                crossInfoPO.setIp(vIntersectionView.getCCONTROLLERIP());
                String[] array = {String.valueOf(vIntersectionView.getLongitude()), String.valueOf(vIntersectionView.getLatitude())};
                crossInfoPO.setLocation(StringUtils.join(array, ","));
                crossInfoPO.setVersion(vIntersectionView.getCControllerModel());
                crossInfoPO.setManufacturerId(22);
                crossInfoPOS.add(crossInfoPO);
            }
        }
        return crossInfoPOS;
    }

    @Override
    public SchemePhaseLightsVO schemePhaseLights(SchemePhaseLightsDTO schemePhaseLightsDTO) throws Exception {
        String crossId = schemePhaseLightsDTO.getCrossId();
        Map<String, CrossInfoPO> crossInfoCache = CrossInfoCache.getCrossInfoCache();
        CrossInfoPO crossInfoPO = crossInfoCache.get(crossId);

        List<CrossLightsPO> crossLightsList = new ArrayList<>();
        List<CrossPhasePO> crossPhaseList = new ArrayList<>();
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = new ArrayList<>();
        List<CrossSchemePO> crossSchemeList = new ArrayList<>();

        List<VNtcipChannelPhaseDTO> vNtcipChannelPhaseDTOS = vNtcipChannelViewMapper.selectChannelPhaseList(crossInfoPO.getCode());
        if (!CollectionUtils.isEmpty(vNtcipChannelPhaseDTOS)) {
            for (VNtcipChannelPhaseDTO vNtcipChannelPhaseDTO : vNtcipChannelPhaseDTOS) {
                // 1灯组信息
                // 通道号
                Integer nChannelControlType = vNtcipChannelPhaseDTO.getNChannelControlType();
                Integer nChannelNumber = vNtcipChannelPhaseDTO.getNChannelNumber();
                Integer nChannelDim = vNtcipChannelPhaseDTO.getNChannelDim();
                CrossLightsPO crossLightsPO = getCrossLights(crossId, nChannelControlType, nChannelNumber, nChannelDim);            crossLightsList.add(crossLightsPO);;
                crossLightsList.add(crossLightsPO);;


                // 环相位信息
                Integer nPhaseConCurrency = vNtcipChannelPhaseDTO.getNPhaseConCurrency();
                String format = String.format("%16s", Integer.toBinaryString(nPhaseConCurrency)).replace(' ', '0');

                // 2相位灯组信息
                CrossPhaseLightsPO crossPhaseLightsPO = getCrossPhaseLightsPO(crossId, vNtcipChannelPhaseDTO, nChannelNumber);
                crossPhaseLightsPOList.add(crossPhaseLightsPO);

                // 3
                CrossSchemePO crossSchemePO = new CrossSchemePO();
                crossSchemePO.setCrossId(crossId);
                crossSchemePO.setSchemeNo(null);
                crossSchemePO.setName(null);
                crossSchemePO.setCycle(null);
                crossSchemePO.setCoordPhase(null);
                crossSchemePO.setOffset(null);
                crossSchemePO.setSource(2);
                crossSchemePO.setStatus(0);
            }
        }
        log.error("======" + crossLightsList);
        return null;
    }

    @NotNull
    private static CrossPhaseLightsPO getCrossPhaseLightsPO(String crossId, VNtcipChannelPhaseDTO vNtcipChannelPhaseDTO, Integer nChannelNumber) {
        CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
        crossPhaseLightsPO.setPhaseId(vNtcipChannelPhaseDTO.getNPhaseNumber());
        CrossLightsPO selectEntity = new CrossLightsPO();
        selectEntity.setCrossId(crossId);
        selectEntity.setLightsNo(String.valueOf(nChannelNumber));
        crossPhaseLightsPO.setLightsId(null);
        crossPhaseLightsPO.setCrossId(crossId);
        return crossPhaseLightsPO;
    }

    private static CrossLightsPO getCrossLights(String crossId, Integer nChannelControlType, Integer nChannelNumber, Integer nChannelDim) {
        CrossLightsPO crossLightsPO = new CrossLightsPO();
        crossLightsPO.setLightsNo(String.valueOf(nChannelNumber));
        crossLightsPO.setName("海信灯组" + nChannelNumber);
        if (Objects.equals(ChannelControlEnum.PEDESTRIAN.getCode(), nChannelControlType)) {
            crossLightsPO.setType(3);
        }
        if (Objects.equals(ChannelControlEnum.VEHICLE.getCode(), nChannelControlType)) {
            crossLightsPO.setType(1);
        }
        String dirTurn = String.format("%8s", Integer.toBinaryString(nChannelDim)).replace(' ', '0');
        if (StringUtils.isNotBlank(dirTurn)) {
            String dir = dirTurn.substring(0, 3); // 高三位 方向
            String turn = dirTurn.substring(3); // 低五位 转向
            crossLightsPO.setDir(HiseseDirEnum.getWjLightsDir(Integer.parseInt(dir, 2)));
            crossLightsPO.setTurn(HisenseTurnEnum.getWjLightsTurn(Integer.parseInt(turn, 2)));
            crossLightsPO.setCrossId(crossId);
        }
        return crossLightsPO;
    }

    @Override
    public List<PlanSectionVO> planSection(PlanSectionDTO planSectionDTO) throws Exception {
        String crossId = planSectionDTO.getCrossId();
        //从缓存中拿路口信息
        CrossInfoPO crossInfoPO = CrossInfoCache.getCrossInfoCache().get(crossId);
        if (Optional.ofNullable(planSectionDTO).isPresent()){
            String code = crossInfoPO.getCode();
            List<VNtcipTimeBaseDayplanDTO> vNtcipTimeBaseDayplanDTOS = vNtcipTimeBaseDayplanMapper.listByCrossIdAndDayPlanId(code, planSectionDTO.getPlanNo());
            if (!CollectionUtils.isEmpty(vNtcipTimeBaseDayplanDTOS)){
                List<PlanSectionVO> list = new ArrayList<>();
                //根据时段表号分组
                Map<Integer, List<VNtcipTimeBaseDayplanDTO>> collect = vNtcipTimeBaseDayplanDTOS.stream().collect(Collectors.groupingBy(VNtcipTimeBaseDayplanDTO::getNDayPlanNumber));
                Set<Integer> keySet = collect.keySet();
                for (Integer i : keySet) {
                    PlanSectionVO planSectionVO = new PlanSectionVO();
                    List<CrossPlanPO> crossPlanPOList = new ArrayList<>();
                    List<CrossSectionPO> crossSectionPOList = new ArrayList<>();
                    //时段数据
                    List<VNtcipTimeBaseDayplanDTO> dayplanDTOS = collect.get(i);
                    for (int i1 = 0; i1 < dayplanDTOS.size(); i1++) {
                        VNtcipTimeBaseDayplanDTO dayplanDTO = dayplanDTOS.get(i1);
                        CrossPlanPO crossPlan = new CrossPlanPO();
                        CrossSectionPO crossSectionPO = new CrossSectionPO();
                        crossPlan.setCrossId(crossId);
                        //计划号-取的是时段表号
                        crossPlan.setPlanNo(dayplanDTO.getNDayPlanNumber()+"");
                        crossPlan.setName("海信信号机-时段表号:"+dayplanDTO.getNDayPlanNumber());

                        crossSectionPO.setCrossId(crossId);
                        //时段号 - 取的是时段序号
                        crossSectionPO.setSectionNo(dayplanDTO.getNDayPlanEventNumber()+"");
                        //TODO 开始时间 - 拼接小时和分钟
                        crossSectionPO.setStartTime(dayplanDTO.getNDayPlanHour()+":"+dayplanDTO.getNDayPlanMinute());
                        if (i1 == dayplanDTOS.size()-1){
                            crossSectionPO.setEndTime("23:59");
                        }else {
                            VNtcipTimeBaseDayplanDTO dto = dayplanDTOS.get(i1 + 1);
                            crossSectionPO.setEndTime(dto.getNDayPlanHour()+":"+dto.getNDayPlanMinute());
                        }
                        // 计划ID - 取的是时段表号
                        crossSectionPO.setPlanId(dayplanDTO.getNDayPlanNumber());
                        //方案号
                        Integer pattern = dayplanDTO.getNTimebaseAscPattern();
                        if (pattern == 85){
                            //黃闪
                            crossSectionPO.setControlMode(3);
                        }else {
                            //定周期
                            crossSectionPO.setControlMode(1);
                        }
                        //方案ID
                        crossSectionPO.setSchemeId(pattern);
                        //创建时间
                        crossSectionPO.setGmtCreate(null);
                        //创建时间
                        crossSectionPO.setGmtModified(null);

                        crossPlanPOList.add(crossPlan);
                        crossSectionPOList.add(crossSectionPO);

                    }
                    planSectionVO.setCrossPlanPOList(crossPlanPOList);
                    planSectionVO.setCrossSectionPOList(crossSectionPOList);
                    list.add(planSectionVO);
                }
                return list;

            }
        }
        return null;
    }

    @Override
    public List<CrossSchedulesPO> crossSchedules(CrossSchedulesDTO crossSchedulesDTO) throws Exception {

        // 路口编号列表
        List<String> crossIdList = crossSchedulesDTO.getCrossIdList();
        // 信号机编号
        List<String> crossCodeList = crossInfoMapper.selectCrossCodesByIds(crossIdList);
        List<CrossSchedulesPO> results = new ArrayList<>(crossCodeList.size());
        if (!CollectionUtils.isEmpty(crossCodeList)) {
            for (String code : crossCodeList) {
                CrossInfoPO crossInfoBySignalCode = CrossInfoCache.getCrossInfoBySignalCode(code);
                String crossId = crossInfoBySignalCode.getId();
                LambdaQueryWrapper<VNtcipTimeBaseScheduleView> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(VNtcipTimeBaseScheduleView::getCIntsID, code);
                List<VNtcipTimeBaseScheduleView> vNtcipTimeBaseScheduleViews = vNtcipTimeBaseScheduleMapper.selectList(queryWrapper);
                if (!CollectionUtils.isEmpty(vNtcipTimeBaseScheduleViews)) {
                    for (VNtcipTimeBaseScheduleView vNtcipTimeBaseScheduleView : vNtcipTimeBaseScheduleViews) {
                        if (Objects.nonNull(vNtcipTimeBaseScheduleView)) {
                            CrossSchedulesPO crossSchedulesPO = new CrossSchedulesPO();
                            crossSchedulesPO.setCrossId(crossId);
                            crossSchedulesPO.setScheduleNo(vNtcipTimeBaseScheduleView.getNTimeBaseScheduleNumber());
                            crossSchedulesPO.setName("海信调度" + vNtcipTimeBaseScheduleView.getNTimeBaseScheduleNumber());
                            // 海信月份
                            Integer hisenseMonth = vNtcipTimeBaseScheduleView.getNTimeBaseScheduleMonth();
                            // 星期
                            Integer hisenseWeek = vNtcipTimeBaseScheduleView.getNTimeBaseScheduleDay();
                            // 日期 2147483647
                            Integer hisenseDays = vNtcipTimeBaseScheduleView.getNTimeBaseScheduleDate();
                            // 时段表号
                            Integer hisensePlanId = vNtcipTimeBaseScheduleView.getNTimeBaseScheduleDayPlan();
                            String monthsStr = ScheduleUtil.getSchedule(hisenseMonth.longValue(), 1);
                            String daysStr = ScheduleUtil.getSchedule(hisenseDays.longValue(), 2);
                            String weekStr = ScheduleUtil.getSchedule(hisenseWeek.longValue(), 3);
                            if (StringUtils.isNotBlank(weekStr)) {
                                String[] weekSplit = weekStr.split(",");
                                for (String oneWeek : weekSplit) {
                                    CrossSchedulesPO crossSchedulesPOWeek = new CrossSchedulesPO();
                                    crossSchedulesPOWeek.setCrossId(crossId);
                                    crossSchedulesPOWeek.setScheduleNo(vNtcipTimeBaseScheduleView.getNTimeBaseScheduleNumber());
                                    crossSchedulesPOWeek.setName("海信调度" + vNtcipTimeBaseScheduleView.getNTimeBaseScheduleNumber());
                                    crossSchedulesPOWeek.setWeek(Integer.valueOf(oneWeek));
                                    crossSchedulesPOWeek.setMonths(monthsStr);
                                    crossSchedulesPOWeek.setDays(daysStr);
                                    crossSchedulesPOWeek.setPlanId(hisensePlanId);
                                    results.add(crossSchedulesPOWeek);
                                }
                            } else {
                                // 不执行周调度
                                CrossSchedulesPO crossSchedulesPOMonthDays = new CrossSchedulesPO();
                                crossSchedulesPOMonthDays.setCrossId(crossId);
                                crossSchedulesPOMonthDays.setScheduleNo(vNtcipTimeBaseScheduleView.getNTimeBaseScheduleNumber());
                                crossSchedulesPOMonthDays.setName("海信调度" + vNtcipTimeBaseScheduleView.getNTimeBaseScheduleNumber());
                                crossSchedulesPOMonthDays.setWeek(null);
                                crossSchedulesPOMonthDays.setMonths(monthsStr);
                                crossSchedulesPOMonthDays.setDays(daysStr);
                                crossSchedulesPOMonthDays.setPlanId(0);
                                results.add(crossSchedulesPOMonthDays);
                            }
                        }
                    }
                }
            }
        }
        return results;
    }


    @Override
    public CoordinationStatus crossPlan(String crossId) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(crossId);
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_12);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());

        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        if (StringUtils.isBlank(post)) {
            log.error("请求 5.7. 协调状态 失败");
            return null;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result)) {
            //成功
            String spot = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Spot");
            String CoordPatternStatus = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CoordPatternStatus");
            String CoordCycleStatus = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CoordCycleStatus");
            String CoordSyncStatus = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CoordSyncStatus");
            CoordinationStatus coordinationStatus = CoordinationStatus.builder()
                    .spot(crossId)
                    .coordPatternStatus(CoordPatternStatus)
                    .coordCycleStatus(CoordCycleStatus)
                    .coordSyncStatus(CoordSyncStatus)
                    .build();
            return coordinationStatus;

        } else {
            //失败
            log.error("请求 5.7. 协调状态 失败");
        }

        return null;
    }

    @Override
    public CrossSchemeRings crossSchemeRings(CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(crossSchemeRingsDTO.getCrossId());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_30);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.PATTERN).setText(crossSchemeRingsDTO.getPattern());
        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        if (StringUtils.isBlank(post)) {
            log.error("请求 5.15.根据方案号获取配时方案环图 失败");
            return null;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result)) {
            //成功
            String patternRest = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Pattern");
            String cycleLen = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CycleLen");
            String ringCount = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("RingCount");
            String offset = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Offset");
            List<Element> cycleElementList = postResult.selectNodes(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT + "/CycleList");
            List<Element> phaseElementList = postResult.selectNodes(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT + "/PhaseList");
            CrossSchemeRings crossSchemeRings = new CrossSchemeRings();
            crossSchemeRings.setSpot(crossSchemeRingsDTO.getCrossId());
            crossSchemeRings.setPattern(patternRest);
            crossSchemeRings.setRingCount(ringCount);
            crossSchemeRings.setCycleLen(cycleLen);
            crossSchemeRings.setOffset(offset);

            JSONObject jsonObject = new JSONObject();
            for (Element cycle : cycleElementList) {
                List<Node> elements = cycle.elements();
                for (Node node : elements) {
                    node.getName();
                    jsonObject.put(node.getName(), node.getText());
                }
            }
            List<CrossSchemeRings.Phase> phaseList = new ArrayList<>();
            for (Element el : phaseElementList) {
                CrossSchemeRings.Phase phase = new CrossSchemeRings.Phase();
                phase.setRedTime(el.valueOf("RedTime"));
                phase.setYellowTime(el.valueOf("YellowTime"));
                phase.setGreenTime(el.valueOf("GreenTime"));
                phase.setDirection(el.valueOf("Direction"));
                phase.setMinGTime(el.valueOf("MinGTime"));
                phase.setMaxGTime(el.valueOf("MaxGTime"));
                phase.setChannelDim(el.valueOf("ChannelDim"));
                phase.setConcurrent(el.valueOf("Concurrent"));
                phaseList.add(phase);
            }
            crossSchemeRings.setCycleList(jsonObject);
            crossSchemeRings.setPhaseList(phaseList);
            return crossSchemeRings;

        } else {
            //失败
            log.error("请求 5.15. 方案环图请求 失败");
        }
        return null;
    }
}
