package net.wanji.utc.hisense.util;

import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.hisense.pojo.xml.pojo.messagecontent.HeartBeatRequest;
import net.wanji.utc.hisense.pojo.xml.pojo.messagecontent.LogOnRequest;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import java.io.*;

/**
 * 封装了XML转换成object，object转换成XML的代码
 *
 * @author duanruiming
 * @date 2024/01/08 10:45
 */
@Slf4j
public class XMLUtils {

    /**
     * 将对象直接转换成String类型的 XML输出
     *
     * @param obj
     * @return
     */
    public static String convertToXml(Object obj) throws Exception {
        // 创建输出流
        StringWriter sw = new StringWriter();
        try {
            sw.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n");
            // 利用jdk中自带的转换类实现
            JAXBContext context = JAXBContext.newInstance(obj.getClass());
            Marshaller marshaller = context.createMarshaller();
            // 格式化xml输出的格式
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            // 去掉默认格式报文头
            marshaller.setProperty(Marshaller.JAXB_FRAGMENT, Boolean.TRUE);
            // 将对象转换成输出流形式的xml
            marshaller.marshal(obj, sw);
        } catch (JAXBException e) {
            log.error("实体类对象转换xml失败:", e);
            throw new Exception("实体类对象转换xml失败:", e);
        }
        return sw.toString();
    }

    public static void main(String[] args) throws Exception {

        HeartBeatRequest heartBeatRequest = new HeartBeatRequest();
        HeartBeatRequest.HeartBeatContent heartBeatContent = new HeartBeatRequest.HeartBeatContent();
        //heartBeatRequest.setMessageContent(body);
        heartBeatRequest.setMessageContent(heartBeatContent);

        heartBeatRequest.setSubSystem("Hisense");
        heartBeatRequest.setIsRequest("1");

        System.err.println(convertToXml(heartBeatRequest));

        System.err.println("================");

        LogOnRequest logOnMessageContent = new LogOnRequest();
        LogOnRequest.Content body = new LogOnRequest.Content();
        body.setUser("user");
        body.setPassword("123456");
        logOnMessageContent.setMessageContent(body);

        logOnMessageContent.setSubSystem("Hisense");
        logOnMessageContent.setFlag("0");
        logOnMessageContent.setIsRequest("");

        System.err.println(convertToXml(logOnMessageContent));
    }

    /**
     * 将对象根据路径转换成xml文件
     *
     * @param obj
     * @param path
     */
    public static String convertToXml(Object obj, String path) {
        FileWriter fw = null;
        try {
            // 利用jdk中自带的转换类实现
            JAXBContext context = JAXBContext.newInstance(obj.getClass());
            Marshaller marshaller = context.createMarshaller();
            // 格式化xml输出的格式
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            // 将对象转换成输出流形式的xml
            // 创建输出流
            fw = new FileWriter(path);
            marshaller.marshal(obj, fw);
        } catch (JAXBException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return fw.toString();
    }

    /**
     * 将String类型的xml转换成对象
     *
     * @param clazz
     * @param xmlStr
     * @return
     */
    public static Object convertXmlStrToObject(Class clazz, String xmlStr) throws Exception {
        Object xmlObject = null;
        try {
            JAXBContext context = JAXBContext.newInstance(clazz);
            // 进行将Xml转成对象的核心接口
            Unmarshaller unmarshaller = context.createUnmarshaller();
            StringReader sr = new StringReader(xmlStr);
            xmlObject = unmarshaller.unmarshal(sr);
        } catch (JAXBException e) {
            log.error("xml字符串转化实体类对象失败:", e);
            throw new Exception("xml字符串转化实体类对象失败:", e);
        }
        return xmlObject;
    }

    /**
     * 将file类型的xml转换成对象
     *
     * @param clazz
     * @param xmlPath
     * @return
     */
    public static Object convertXmlFileToObject(Class clazz, String xmlPath) {
        Object xmlObject = null;
        try {
            JAXBContext context = JAXBContext.newInstance(clazz);
            Unmarshaller unmarshaller = context.createUnmarshaller();
            FileReader fr = null;
            fr = new FileReader(xmlPath);
            xmlObject = unmarshaller.unmarshal(fr);
        } catch (JAXBException e) {
            e.printStackTrace();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        return xmlObject;
    }

    public static String strToHex(String str) {
        StringBuilder sb = new StringBuilder();
        for (char c : str.toCharArray()) {
            sb.append(String.format("%02X", (int) c));
        }
        return sb.toString();
    }
}
